;(function(document, $) {
    "use strict";

    var CANCEL_URL = '/services/smartling/submission/bulk/cancel';

    var cancelButton;
    var ui;

    $(document).on("foundation-contentloaded", function(e){
		if (!cancelButton) {
			cancelButton = new CancelButton();
			cancelButton.initialize();
		}

		 ui = $(window).adaptTo("foundation-ui");
	});

	var CancelButton = new Class({

		$cancelButton: null,

		initialize: function () {
			var self = this;
			self.$cancelButton = document.querySelector(".smartling-submission-cancel");

            if (!self.$cancelButton) {
                return;
            }

			Coral.commons.ready(self.$cancelButton, function() {

				self.$cancelButton.on('click', function(event){
				    ui.prompt('Confirmation', 'Translation for selected items and locales will be canceled.', 'notice', [
                        {id: 'smartling-cancel-confirmation', text: 'Yes', handler: self._cancelAction.bind(self)},
                        {id: 'smartling-cancel-declining', text: 'No', primary: true}
                    ]);
				});

			})
		},

		_cancelAction: function() {

            var params = $.param({'cancelSubmissions': JSON.stringify(this._constructCancelSubmissionObjects())});
            var waitTicker = ui.waitTicker('Loading');
            $.post({
                url: CANCEL_URL,
                data: params
            }).done(function(data) {
                ui.alert('Cancel submissions', 'Selected submissions have been cancelled.', 'success');
            }).fail(function(data) {
                ui.alert('Failed to cancel selected submissions.', data.responseJSON.errors + '.', 'error');
            }).always(function(data) {
                waitTicker.clear();
            });
        },

        _constructCancelSubmissionObjects: function() {

            var nodes = [];
            $('tr.is-selected[data-node-path]').each(function(i, item) {
                nodes.push(item.dataset);
            });

            return nodes.map(function(item) {
                return {fileUri: item.fileUri, sourcePath: item.sourcePath, locale: item.locale};
            });
        }
	});
})(document, Granite.$);