(function ($, $document, author) {
    function extendEditableModel(Editable) {
        Editable.prototype.updateSmartlingLockState = function (locked) {
            if (!this.smartlingConfig || this.smartlingConfig.locked !== locked) {
                this.smartlingConfig = this.smartlingConfig || {};
                this.smartlingConfig.locked = locked;

                // This is faster then overlayManager.recreate(this)
                if (locked) {
                    this.overlay.dom.addClass('smartling-locked-element');
                } else {
                    this.overlay.dom.removeClass('smartling-locked-element');
                }
            }
        };

        Editable.prototype.__proto__.getSmartlingLockState = function () {
            return this.smartlingConfig && this.smartlingConfig.locked;
        }
    }

    function extendOverlayRendering(Overlay) {
        var previousOverlayRenderingCode = Overlay.prototype.render;
        Overlay.prototype.render = function (editable) {
            var $dom = previousOverlayRenderingCode.call(this, editable);
            if (editable.getSmartlingLockState()) {
                $dom.addClass('smartling-locked-element');
            }

            return $dom;
        };
    }

    function loadComponentLockingInfo(pagePath) {
        return $.ajax({
            type: 'GET',
            url: '/services/smartling/components/locked',
            data: {
                pagePath: pagePath
            }
        });
    }

    function lockComponents(lockedProperties) {
        // Due to sorting, the most specific components go first in the list
        var editables = author.store.slice(0).sort(function (a, b) {
            return b.path.localeCompare(a.path);
        });

        function toClosestEditable(propertyPath) {
            for (var i = 0; i < editables.length; ++i) {
                if (propertyPath.startsWith(editables[i].path + "/")) {
                    return editables[i];
                }
            }

            return null;
        }

        function notNull(object) {
            return object;
        }

        lockedProperties
            .map(function (relativePath) {return author.page.path + "/" + relativePath;})
            .map(toClosestEditable)
            .filter(notNull)
            .forEach(function (editable) { editable.updateSmartlingLockState(true) });
    }

    extendEditableModel(author.Editable);
    extendOverlayRendering(author.Overlay);

    $document.on('cq-layer-activated', function(event) {
        loadComponentLockingInfo(author.page.path)
            .then(function (data) {
                lockComponents(data.lockedProperties || []);
                return data;
            })
            .done(function (data) {
                if (data.isTarget && event.layer === 'Edit') {
                    SMARTLING.ui.touch.Toolbar.lockComponentAction.register();
                    SMARTLING.ui.touch.Toolbar.unlockComponentAction.register();
                }
            });
    });
})(jQuery, jQuery(document), Granite.author);

