CQ.Ext.ns('SMARTLING.ui.sidekick');

//TODO: These functions must go to a global namespace and be reused
function previewDoc(targetPath) {
    if (targetPath === null) {
        CQ.Ext.Msg.show({
            title: 'Warning',
            msg: 'Preview is not available',
            icon: CQ.Ext.Msg.INFO,
            buttons: CQ.Ext.Msg.OK
        });
    }
    else {
        window.open('/apps/smartling/preview.html?source=' + CQ.WCM.getPagePath() + '&target=' + targetPath);
    }
}

function downloadTranslations(submission) {
    CQ.Ext.Msg.confirm("Confirmation", "Are you sure you want to proceed?", function (btn) {
        if (btn === 'yes') {
            CQ.Ext.Ajax.request({
                method: 'POST',
                url: '/services/smartling/submission/bulk/download',
                params: {
                    nodePaths: [submission.nodePath]
                },
                callback: function (options, success) {
                    CQ.Ext.Msg.show({
                        title: 'Download',
                        msg: success ? 'Successfully downloaded.' : 'Failed to download translations.',
                        width: 300,
                        buttons: CQ.Ext.Msg.OK,
                        multiline: false
                    });
                }
            });
        }
    });
}

SMARTLING.ui.sidekick.SmartlingPageStatusStore = CQ.Ext.extend(CQ.Ext.data.JsonStore, {
    constructor: function (config) {
        CQ.Util.applyDefaults(config, {
            proxy: new CQ.Ext.data.HttpProxy({
                api: {
                    read: {url: '/services/smartling/submission/pageStatus.json', method: 'GET'}
                },
                headers: {"Content-Type": "application/json; charset=utf-8"}
            }),
            baseParams: {path: CQ.WCM.getPagePath()},
            root: 'records',
            fields: [
                'targetPath',
                'languageName',
                'pageStatus',
                'pageStatusIcon',
                'progress',
                'nodePath',
                'previewTranslation',
                'locale'
            ],
            totalProperty: 'results'
        });
        SMARTLING.ui.sidekick.SmartlingPageStatusStore.superclass.constructor.call(this, config);
    }
});

SMARTLING.ui.sidekick.SmartlingGrid = CQ.Ext.extend(CQ.Ext.grid.GridPanel, {
    constructor: function (config) {

        function canBeDownloaded(pageStatus) {
            return (pageStatus === 'red' || pageStatus === 'blue')
        }

        var cmCfg = {
            defaults: {
                sortable: false,
                menuDisabled: true
            },
            columns: [
                {
                    header: "<font size='1'>Locale</font>",
                    dataIndex: 'languageName',
                    width: 100
                },
                {
                    xtype: 'templatecolumn',
                    dataIndex: 'pageStatusIcon',
                    header: "<font size='1'>Status</font>",
                    width: 40,
                    tpl: '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-{pageStatusIcon}.gif">'
                },
                {
                    dataIndex: 'progress',
                    header: "<font size='1'>Progress</font>",
                    width: 55,
                    renderer: function (value) {
                        if (value == 2) {
                            return '';
                        } else {
                            var id = CQ.Ext.id();
                            (function () {
                                new CQ.Ext.ProgressBar({
                                    renderTo: id,
                                    value: value
                                });
                            }).defer(25);
                            return '<div id="' + id + '"></div>';
                        }
                    }
                }
            ]
        };

        var actionItems = [];
        if (config.actions.download) {
            actionItems.push({
                tooltip: 'Download Translation',
                icon: '/libs/cq/ui/widgets/themes/default/icons/16x16/down.gif',
                handler: function (grid, rowIndex) {
                    var rec = grid.getStore().getAt(rowIndex);
                    downloadTranslations(rec.data);
                },
                getClass: function (value, metadata, record) {
                    return canBeDownloaded(record.data.pageStatus) ? '' : 'x-hide-display';

                }
            });
        }
        if (config.actions.preview) {
            actionItems.push({
                tooltip: 'Preview',
                icon: '/libs/cq/ui/widgets/themes/default/widgets/wcm/Sidekick/preview.png',
                handler: function (grid, rowIndex) {
                    var rec = grid.getStore().getAt(rowIndex);
                    previewDoc(rec.data.targetPath)
                },
                getClass: function (value, metadata, record) {
                    return record.data.previewTranslation ? '' : 'x-hide-display';
                }
            })
        }
        if (actionItems.length > 0) {
            cmCfg.columns.push({
                dataIndex: 'nodePath',
                width: 50,
                xtype: 'actioncolumn',
                items: actionItems
            });
        }

        CQ.Util.applyDefaults(config, {
            border: false,
            autoHeight: true,
            loadMask: true,
            store: new SMARTLING.ui.sidekick.SmartlingPageStatusStore({autoLoad: true}),
            cm: new CQ.Ext.grid.ColumnModel(cmCfg)
        });
        SMARTLING.ui.sidekick.SmartlingGrid.superclass.constructor.call(this, config);
    }
});

SMARTLING.ui.sidekick.SmartlingTranslationPanel = function (sidekick, config) {
    CQ.Ext.applyIf(config, {actions: {}});

    var translationStatusGrid = new SMARTLING.ui.sidekick.SmartlingGrid(config);

    var panelConfig = {
        "title": "Smartling Translations",
        "layout": "form",
        "labelAlign": "top",
        "cls": "cq-sidekick-form",
        "items": [
            {
                "xtype": "panel",
                "ctCls": "x-form-element-label-top",
                "border": false,
                "fieldLabel": CQ.I18n.getMessage("Select Languages"),
                "items": [translationStatusGrid]
            },
            {
                "xtype": "panel",
                "layout": "form",
                "border": false,
                "cls": "cq-sidekick-form",
                "defaults": {
                    "border": false
                },
                "items": [
                    {
                        "xtype": "label",
                        "fieldLabel": "* Source modified since last submitted"
                    },
                    {
                        "xtype": "label",
                        "fieldLabel": '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-red.gif"> In Translation'
                    },
                    {
                        "xtype": "label",
                        "fieldLabel": '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-blue.gif"> Complete'
                    },
                    {
                        "xtype": "label",
                        "fieldLabel": '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-empty.gif"> Canceled'
                    },
                    {
                        "xtype": "label",
                        "fieldLabel": '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-skipped.gif"> Failed'
                    },
                    {
                        "xtype": "label",
                        "fieldLabel": '<img style="vertical-align:text-bottom;" src="/libs/cq/ui/widgets/themes/default/icons/16x16/bulb-grey.gif"> Not sent'
                    }
                ]
            }
        ]
    };

    if (config.actions.translate) {
        panelConfig.items.splice(1, 0, {
            "xtype": "panel",
            "layout": "form",
            "border": false,
            "cls": "cq-sidekick-form",
            "defaults": {
                "border": false
            },
            "items": [
                {
                    "xtype": "button",
                    "text": CQ.I18n.getMessage("Translate"),
                    "handler": function () {
                        var dialog = new SMARTLING.ui.dialog.SendForTranslationDialog({paths: [CQ.WCM.getPagePath()]});
                        dialog.show();
                    }
                }
            ]
        });
    }

    return new CQ.Ext.Panel(panelConfig);
};