;(function ($, smartlingApi) {

    //Class, responsible for rendering progress bar content in sidekick panel.
    SmartlingApi.ProgressBar = new Class({

        /**
         * notifications/mask/unmask support
         */
        foundationUi: null,

        /**
         * jQuery object, representing root element, where the ProgressBar should be rendered
         */
        $contentPanel: null,

        /**
         * The URL from which the content of ProgressBar should be loaded (produces HTML markup)
         */
        loadUrl: null,

        construct: function ($element) {
            this.$contentPanel = $element;
            this.$translateButton = $('#classic-smartling-job-trigger');

            this.foundationUi = $(window).adaptTo('foundation-ui');

            if($element.is('[src]')) { // if element has src attribute defined
                this.loadUrl = $element.attr('src') + '.html';
            }

            this._initButtonListeners();

            var self = this;
            $(document).on('progressBar:reload', function() {
                self._wait();
                setTimeout(function() {
                    self.load(smartlingApi.getContentPath());
                }, 3000);
            });
        },

        /**
         * Load ProgressBar contents from server and render the result.
         * @param contentPath translatable resource, for which progress status should be displayed inside the progress bar
         */
        load: function (contentPath) {
            var self = this;
            if(!self.loadUrl) {
                self._notify('Failed to retrieve translation status', 'error');
                return;
            }

            self._wait();
            $.ajax({
                url: self.loadUrl + '?path=' + contentPath
            }).done(function (data) {
                self.$contentPanel.html(data);
                self._showTranslateButton();
            }).fail(function () {
                $('coral-tab[icon=smartlingTranslationsPanel]').hide()
            }).always(function () {
                self._clearWait();
            });
        },

        /**
         * Clear the contents of ProgressBar, replacing it with specified HTML snippet, if provided
         * @param message optional message (may include html markup) to be displayed on $contentPanel (defaults to '').
         */
        clear: function (message) {
            this._hideTranslateButton();
            this.$contentPanel.html('<div class="smartling-progress-panel-message">' + (message || '') + '</div>');
        },

        /**
         * PRIVATE
         */

        /**
         * Initialize action buttons on translation-progress-panel.
         * @private
         */
        _initButtonListeners: function () {
            var self = this;

            this.$contentPanel.on('click', 'button.js-btn-preview', function (e) {
                e.preventDefault();
                var targetPath = $(this).closest('.js-smartling-sidepanel-list-row').data('targetPath');
                var sourcePath = smartlingApi.getContentPath(); // TODO: don't use smartlingApi here, path should be passed as external parameter

                if (!!!sourcePath || !!!targetPath) {
                    self._notify('Failed to calculate preview path', 'error');
                    return;
                }

                var url = '/mnt/overlay/wcm/core/content/sites/diffresources.html' + sourcePath + '?item=' + targetPath + '&sideBySide';
                window.open(url, '_blank');
            });

            this.$contentPanel.on('click', 'button.js-btn-download', function (e) {
                e.preventDefault();

                self._wait();
                $.ajax({
                    type: 'POST',
                    url: '/services/smartling/submission/bulk/download',
                    data: {
                        nodePaths: [$(this).closest('.js-smartling-sidepanel-list-row').data('nodePath')]
                    }
                }).then(function (data) {
                    if (data.success) {
                        self._notify('Translations downloaded', 'success');
                    } else {
                        self._notify('Failed to download translations', 'error');
                    }
                }).fail(function () {
                    self._notify('Failed to download translations', 'error');
                }).always(function () {
                    self._clearWait();
                    $(document).trigger('progressBar:reload');
                });
            });

            this.$translateButton.click(function () {
                var $dialog = $('#classic-smartling-job-dialog');
                $dialog.trigger("smartlingTranslateDialog:open");
                $dialog.get(0).show();
            });
        },

        _showTranslateButton: function() {
            this.$translateButton.show();
        },

        _hideTranslateButton: function() {
            this.$translateButton.hide();
        },

        /**
         * Mask the ProgressBar element.
         * @private
         */
        _wait: function () {
            this.foundationUi.wait(this.$contentPanel);
        },

        /**
         * Remove the mask from ProgressBar element.
         * @private
         */
        _clearWait: function () {
            this.foundationUi.clearWait();
        },

        /**
         * Display page-wide toast notification with specified message and type.
         *
         * @param message text to be displayed (may include html markup)
         * @param type 'success', 'error' or 'info'
         * @private
         */
        _notify: function (message, type) {
            this.foundationUi.notify(type, message, type);
        }
    });

})(Granite.$, window.SmartlingApi = window.SmartlingApi || {});